/* -*- Mode:C++; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */
/*
 * Copyright (c) 2009-2010, Uwicore Laboratory (www.uwicore.umh.es),
 *                          University Miguel Hernandez, EU FP7 iTETRIS project
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Author: Ramon Bauza <rbauza@umh.es>
 */

#ifndef BUILDING_MAP_MODEL_H
#define BUILDING_MAP_MODEL_H

#include "ns3/visibility-model.h"
#include "ns3/mobility-model.h"
#include "ns3/object.h"
#include "ns3/clwpr-map.h"
#include "ns3/pointer.h"

namespace ns3 {

class Segment;

/**
 * @class BuildingMapModel
 * @brief This class provides the LOS/NLOS visibility between tx and rx based on the information obtained from a building map file that provides the positions of the walls of the buildings (x1,y1,x2,y2) present in the scenario
 */

class BuildingMapModel : public VisibilityModel
{
public:
    
    typedef std::vector< Ptr<Segment> >::const_iterator Iterator;

    static TypeId GetTypeId (void);
    BuildingMapModel (void);
    ~BuildingMapModel ();
    BuildingMapModel (std::string path, Ptr<GridMap> map);
    BuildingMapModel (std::string path);
    bool GetVisibility (Ptr<MobilityModel> a, Ptr<MobilityModel> b) const;
    void GetNlosDistances(Ptr<MobilityModel> a, Ptr<MobilityModel> b, double &dist1, double &dist2) const;
    void InitializeVisibilityModel (void);
    void SetGridMap (Ptr<GridMap> map);

    static uint32_t Add (Ptr<Segment> segment);
    static Iterator Begin (void);
    static Iterator End (void);
    static Ptr<Segment> GetSegment (uint32_t n);
    static uint32_t GetNSegments (void);
    static std::vector<Ptr<Segment> > GetSegments (void);
    static bool inLineOfSight (Vector a, Vector b);
    
//private:
  bool DoSegmentsIntersect (Ptr<Segment> seg1, Ptr<Segment> seg2) const;
  Ptr<GridMap> m_map;
};

} // namespace ns3

#endif /* BUILDING_MAP_MODEL_H */
