#include "segment.h"
#include "building-map-model.h"

#include <ns3/enum.h>
#include <ns3/uinteger.h>
#include <ns3/log.h>
#include <ns3/assert.h>
#include <cmath>

NS_LOG_COMPONENT_DEFINE ("Segment");

namespace ns3 {


NS_OBJECT_ENSURE_REGISTERED (Segment);

TypeId
Segment::GetTypeId (void)
{
  static TypeId tid = TypeId ("ns3::Segment")
    .SetParent<Object> ()
    .AddConstructor<Segment> ()
    .AddAttribute ("Id", "The id (unique integer) of this Segment.",
                   UintegerValue (0),
                   MakeUintegerAccessor (&Segment::GetId),
                   MakeUintegerChecker<uint32_t> ())
  ;  
  return tid;
}

Segment::Segment ()
{
  NS_LOG_FUNCTION (this); 
}

Segment::Segment (double x1,
                  double y1,
                  double x2,
                  double y2)
:
        x1(x1),
        y1(y1),
        x2(x2),
        y2(y2)
{
  NS_LOG_FUNCTION (this);
  m_segmentId = BuildingMapModel::Add(this);
}

Segment::~Segment () 
{
  NS_LOG_FUNCTION (this);
}

void
Segment::DoDispose () 
{
  NS_LOG_FUNCTION (this);
}

uint32_t
Segment::GetId (void) const
{
  NS_LOG_FUNCTION (this);
  return m_segmentId;
}

}